@@ -16,6 +16,7 @@
 #include <assert.h>
 
 #include <common/mi-lttng.h>
+#include <lttng/domain-internal.h>
 
 #include "../command.h"
 
@@ -225,7 +226,7 @@ static int disable_events(char *session_name, char *event_list)
 			enabled = 0;
 			success = 1;
 			MSG("All %s events of type %s are disabled in channel %s",
-					get_domain_str(dom.type),
+					lttng_domain_type_str(dom.type),
 					print_event_type(opt_event_type),
 					print_channel_name(channel_name));
 		}
@@ -264,7 +265,7 @@ static int disable_events(char *session_name, char *event_list)
 				enabled = 1;
 			} else {
 				MSG("%s %s of type %s disabled in channel %s for session %s",
-						get_domain_str(dom.type),
+						lttng_domain_type_str(dom.type),
 						event_name,
 						print_event_type(opt_event_type),
 						print_channel_name(channel_name),
